import response_codes as codes
import utility_integration as util
import sys

sys.path.append("../../../../tests")

class test_access_control_webui(util.WrapTest):
    def test_access_control_webui_base_functionality(self):
        base_url = "/system/access_control/webui/config"
        with self.subTest("configure_section"):
            x = self.put_data(base_url + "/general", {
                "enable_http":"1",
                "enable_https":"1",
                "redirect_https":"0",
                "_httpWanAccess":"0",
                "listen_http":"80",
                "_httpsWanAccess":"0",
                "listen_https":"443",
                "rfc1918_filter":"1",
                "device_files":"1",
                "cert":"/etc/uhttpd.crt",
                "key":"/etc/uhttpd.key"
            })
            x.assert_data({
                "id":"general",
                ".type":"uhttpd",
                "enable_http":"1",
                "enable_https":"1",
                "redirect_https":"0",
                "_httpWanAccess":"0",
                "listen_http":"80",
                "_httpsWanAccess":"0",
                "listen_https":"443",
                "rfc1918_filter":"1",
                "device_files":"1",
                "cert":"/etc/uhttpd.crt",
                "key":"/etc/uhttpd.key"
            }, 200, {"cert:file_size", "key:file_size"})
        with self.subTest("single_get"):
            x = self.get(base_url + "/general")
            x.assert_data({
                "id":"general",
                ".type":"uhttpd",
                "enable_http":"1",
                "enable_https":"1",
                "redirect_https":"0",
                "_httpWanAccess":"0",
                "listen_http":"80",
                "_httpsWanAccess":"0",
                "listen_https":"443",
                "rfc1918_filter":"1",
                "device_files":"1",
                "cert":"/etc/uhttpd.crt",
                "key":"/etc/uhttpd.key"
            }, 200, {"cert:file_size", "key:file_size"})
        with self.subTest("multiple_get"):
            x = self.get(base_url)
            x.assert_data([{
                "id":"general",
                ".type":"uhttpd",
                "enable_http":"1",
                "enable_https":"1",
                "redirect_https":"0",
                "_httpWanAccess":"0",
                "listen_http":"80",
                "_httpsWanAccess":"0",
                "listen_https":"443",
                "rfc1918_filter":"1",
                "device_files":"1",
                "cert":"/etc/uhttpd.crt",
                "key":"/etc/uhttpd.key"
            }], 200, {"cert:file_size", "key:file_size"})
        with self.subTest("clear_configuration"):
            x = self.put_data(base_url + "/general", {
                "enable_http":"1",
                "enable_https":"1",
                "redirect_https":"",
                "_httpWanAccess":"",
                "listen_http":"80",
                "_httpsWanAccess":"",
                "listen_https":"443",
                "rfc1918_filter":"1",
                "device_files":"1",
                "cert":"/etc/uhttpd.crt",
                "key":"/etc/uhttpd.key"
            })
            x.assert_data({
                ".type":"uhttpd",
                "id":"general",
                "enable_http":"1",
                "enable_https":"1",
                "listen_http":"80",
                "listen_https":"443",
                "rfc1918_filter":"1",
                "device_files":"1",
                "cert":"/etc/uhttpd.crt",
                "key":"/etc/uhttpd.key"
            }, 200, {"cert:file_size", "key:file_size"})
        with self.subTest("test_ac_webui_deletion"):
            x = self.delete(base_url)
            x.assert_error("Validation", "Section deletion is not allowed", 111, None, None)
        with self.subTest("test_ac_webui_creation"):
            x = self.post_data(base_url, {})
            x.assert_error("Validation", "Section creation is not allowed", 108, None, None)
