import sys
sys.path.append("../../../../tests")
from utility_integration import WrapTest, Env
from utils.general_api import get_mobile_ids
import response_codes as codes

RC = codes.ResponseCodes

class Azure(WrapTest):
    url = "/services/cloud_solutions/azure_iothub/config/general"

    empty = {
        "enable": "",
        "mqtt_topic": "",
        "mqtt_port": "",
        "id": "",
        "msg_type": "",
        ".type": "azure_iothub",
        "mqtt_ip": "",
        "connection_string": "",
        "mqtt_username": "",
        "mqtt_password": "",
        "message_interval": "",
        "interface": "",
        "gsm": ""
    }

    mqtt = {
        "enable": "0",
        "mqtt_topic": "test",
        "mqtt_topic": "test",
        "mqtt_port": "69",
        "enable": "0",
        "id": "general",
        "msg_type": "mqtt",
        ".type": "azure_iothub",
        "mqtt_ip": "example.com",
        "connection_string": "test_connecntion_string",
        "mqtt_username": "test_username",
        "mqtt_password": "test_password"
    }

    gsm = {
        "enable": "0",
        "id": "general",
        ".type": "azure_iothub",
        "msg_type": "gsmctl",
        "gsm": [
            "rsrq",
            "pinstate",
            "manuf",
            "bsent",
            "revision",
            "ecio",
            "sinr",
            "operator",
            "model",
            "imei",
            "imsi",
            "ipaddr",
            "iccid",
            "connstate",
            "cellid",
            "signal",
            "conntype",
            "rsrp",
            "netstate",
            "opernum",
            "serial",
            "brecv",
            "rscp",
            "pincount",
            "temp",
            "simstate"
        ],
        "connection_string": "asdadasd",
        "message_interval": "300",
        "interface": ""
    }

    def setUp(self):
        x = self.put_data(self.url, self.empty)
        x.assert_data({
            "id": "general",
            ".type": "azure_iothub"
        }, 200)

    def test_mqtt_option(self):

        with self.subTest("update_with_mqtt_config"):
            x = self.put_data(self.url, self.mqtt)
            x.assert_data(self.mqtt, 200)

        with self.subTest("clear_config"):
            x = self.put_data(self.url, self.empty)
            x.assert_data({
                "id": "general",
                ".type": "azure_iothub"
            }, 200)

    def test_gsm_option(self):

        mobile_ids = get_mobile_ids(self)

        if len(mobile_ids) == 0:
            self.skipTest("requires modem")

        self.gsm["interface"] = mobile_ids[0]

        with self.subTest("update_with_gsm_config"):
            x = self.put_data(self.url, self.gsm)
            x.assert_data(self.gsm, 200)

        with self.subTest("update_with_bad_gsm_config"):
            self.gsm["gsm"] = ["test"]
            error_msg = "Must be one of the following values [brecv, bsent, cellid, pincount, pinstate, revision, rscp, rsrp, rsrq, serial, signal, " \
                        "simstate, sinr, connstate, conntype, ecio, iccid, imei, imsi, ipaddr, manuf, model, netstate, operator, opernum, temp]."
            x = self.put_data(self.url, self.gsm)
            x.assert_code(422)
            x.assert_error("gsm", error_msg, RC.INVALID_OPT.val())

        with self.subTest("clear_config"):
            x = self.put_data(self.url, self.empty)
            x.assert_data({
                "id": "general",
                ".type": "azure_iothub"
            }, 200)