import sys
import utility_integration as util
from utils.ssh import open_ssh_connection, send_cmd
from time import sleep
sys.path.append("../../../../tests")

class test_certificates(util.WrapTest):

    def test_certificates_base_functionality(self):
        self.maxDiff = None
        base_url = "/system/certificates"
        with self.subTest("start_simple_generation"):
            x = self.post_data(base_url + "/actions/generate", {
                "days":"3560",
                "delete":"0",
                "sign":"0",
                "subject":"",
                "type":"simple"
            })
            x.assert_data({
                0
            })
        with self.subTest("get_dh"):
            x = self.get(base_url + "/config/dh")
            x.assert_data([])
        with self.subTest("kill_dh_generation"):
            sleep(5)
            with open_ssh_connection() as ssh:
                generation_pid = send_cmd(ssh, "ps -w | grep [o]penssl | grep dh.pem | awk {'print$1'}")
                while generation_pid.isdigit() != True:
                    sleep(1)
                    generation_pid = send_cmd(ssh, "ps -w | grep [o]penssl | grep dh.pem | awk {'print$1'}")
                    generation_pid = generation_pid.strip()
                send_cmd(ssh, "kill " + generation_pid)
        with self.subTest("get_all"):
            x = self.get(base_url + "/config")
            resp = x.resp.json()
            generated = resp["data"]["generated"]
            body = [
                {
                    "type": "key",
                    "name": "ca",
                    "cert_type": "ca",
                    "key_size": "2048",
                    "fullname": "ca.key.pem"
                },
                {
                    "type": "cert",
                    "name": "ca",
                    "cert_type": "ca",
                    "key_size": "2048",
                    "fullname": "ca.cert.pem"
                },
                {
                    "type": "key",
                    "name": "server",
                    "cert_type": "server",
                    "key_size": "2048",
                    "fullname": "server.key.pem"
                },
                {
                    "type": "cert",
                    "name": "server",
                    "cert_type": "server",
                    "key_size": "2048",
                    "fullname": "server.cert.pem"
                },
                {
                    "type": "key",
                    "name": "client",
                    "cert_type": "client",
                    "key_size": "2048",
                    "fullname": "client.key.pem"
                },
                {
                    "type": "cert",
                    "name": "client",
                    "cert_type": "client",
                    "key_size": "2048",
                    "fullname": "client.cert.pem"
                }
            ]
            for section in generated:
                inserted = False
                for sectionn in body:
                    if inserted == True: continue
                    if not "timestamp" in sectionn:
                        sectionn["timestamp"] = section["timestamp"]
                        inserted = True
            self.assertEqual(generated, body)

        with self.subTest("get_keys"):
            x = self.get(base_url + "/config/keys")
            x.assert_data([
                {
                    "type": "key",
                    "name": "ca",
                    "timestamp": "-",
                    "cert_type": "ca",
                    "key_size": "2048",
                    "fullname": "ca.key.pem"
                },
                {
                    "type": "key",
                    "name": "server",
                    "timestamp": "-",
                    "cert_type": "server",
                    "key_size": "2048",
                    "fullname": "server.key.pem"
                },
                {
                    "type": "key",
                    "name": "client",
                    "timestamp": "-",
                    "cert_type": "client",
                    "key_size": "2048",
                    "fullname": "client.key.pem"
                }
            ])
        with self.subTest("get_client"):
            x = self.get(base_url + "/config/client")
            resp = x.resp.json()
            keys = resp["data"]["keys"]
            certificates = resp["data"]["certificates"]
            keys_body = [
                    {
                        "type": "key",
                        "name": "client",
                        "cert_type": "client",
                        "key_size": "2048",
                        "timestamp":"-",
                        "fullname": "client.key.pem"
                    }
            ]
            certificates_body = [
                    {
                        "type": "cert",
                        "name": "client",
                        "cert_type": "client",
                        "key_size": "2048",
                        "fullname": "client.cert.pem"
                    }
            ]
            self.assertEqual(keys, keys_body)
            certificates_body[0]["timestamp"] = certificates[0]["timestamp"]
            self.assertEqual(certificates, certificates_body)

        with self.subTest("get_server"):
            x = self.get(base_url + "/config/server")
            resp = x.resp.json()
            keys = resp["data"]["keys"]
            certificates = resp["data"]["certificates"]
            keys_body = [
                    {
                        "type": "key",
                        "name": "server",
                        "cert_type": "server",
                        "key_size": "2048",
                        "timestamp":"-",
                        "fullname": "server.key.pem"
                    }
            ]
            certificates_body = [
                    {
                        "type": "cert",
                        "name": "server",
                        "cert_type": "server",
                        "key_size": "2048",
                        "fullname": "server.cert.pem"
                    }
            ]
            self.assertEqual(keys, keys_body)
            certificates_body[0]["timestamp"] = certificates[0]["timestamp"]
            self.assertEqual(certificates, certificates_body)

        with self.subTest("get_ca"):
            x = self.get(base_url + "/config/ca")
            resp = x.resp.json()
            keys = resp["data"]["keys"]
            certificates = resp["data"]["certificates"]
            keys_body = [
                    {
                        "type": "key",
                        "name": "ca",
                        "cert_type": "ca",
                        "key_size": "2048",
                        "timestamp":"-",
                        "fullname": "ca.key.pem"
                    }
            ]
            certificates_body = [
                    {
                        "type": "cert",
                        "name": "ca",
                        "cert_type": "ca",
                        "key_size": "2048",
                        "fullname": "ca.cert.pem"
                    }
            ]
            self.assertEqual(keys, keys_body)
            certificates_body[0]["timestamp"] = certificates[0]["timestamp"]
            self.assertEqual(certificates, certificates_body)
        with self.subTest("get_certificates"):
            x = self.get(base_url + "/config/certificates")
            x.assert_data([
                {
                    "type": "cert",
                    "name": "server",
                    "cert_type": "server",
                    "key_size": "2048",
                    "fullname": "server.cert.pem"
                },
                {
                    "type": "cert",
                    "name": "client",
                    "cert_type": "client",
                    "key_size": "2048",
                    "fullname": "client.cert.pem"
                }
            ], 200, {"timestamp"})