import sys
sys.path.append("../../../../tests")
from utility_integration import WrapTest, Env
from utils.general_api import get_mnfinfo
from utils.ssh import open_ssh_connection, ubus_call
from time import sleep
from datetime import datetime

class test_cloud_solutions(WrapTest):
    url = "/services/cloud_solutions/rms/config/general"
    status = "/services/cloud_solutions/rms/status"

    empty = {
        "enable": "",
        "remote": "",
        "port": ""
    }

    default = {
        "enable": "0",
        "remote": "rms.teltonika-networks.com",
        "port": "15009",
        ".type": "rms_connect_mqtt"
    }

    update = {
        "enable": "2",
        "remote": "rms.com",
        "port": "6969",
        ".type": "rms_connect_mqtt"
    }

    def setUp(self):
        x = self.put_data(self.url, self.empty)
        x.assert_data({
            "id": "general",
            ".type": "rms_connect_mqtt"
        }, 200)

    def test_basic(self):

        with self.subTest("set_to_modify_values"):
            x = self.put_data(self.url, self.update)
            x.assert_data({**self.update, "id": "general"}, 200)

        with self.subTest("set_to_default_values"):
            x = self.put_data(self.url, self.default)
            x.assert_data({**self.default, "id": "general"}, 200)

    def test_status(self):

        mnfinfo = get_mnfinfo(self)

        with self.subTest("set_to_default_values"):
            x = self.put_data(self.url, self.default)
            x.assert_data({**self.default, "id": "general"}, 200)

        with self.subTest("check_default_config_status"):
            changed_status = False
            started_time = datetime.now()
            
            while ((datetime.now() - started_time).total_seconds()) < 5:
                x = self.get(self.status)
                data = x.resp.json()["data"]
                if not ("error" in data):
                    x.assert_data({
                        "lan_mac": mnfinfo["mac"],
                        "serial_nbr" : mnfinfo["serial"],
                        "status": "0"
                    }, 200)
                    changed_status = True
                    break
                sleep(0.1)

            if not changed_status:
                self.fail("Connection state does not change to 'disabled'")
            

        with self.subTest("check_connect_action"):
            self.default["enable"] = "1"
            x = self.put_data(self.url, self.default)
            x.assert_data({**self.default, "id": "general"}, 200)

            changed_status = False
            started_time = datetime.now()
            
            while ((datetime.now() - started_time).total_seconds()) < 5:
                x = self.get(self.status)
                data = x.resp.json()["data"]
                if "msg" in data and data["msg"] == "Connecting":
                    x.assert_data({
                        "msg": "Connecting",
                        "serial_nbr": mnfinfo["serial"],
                        "lan_mac": mnfinfo["mac"],
                        "next_try": "0",
                        "status": self.default["enable"], 
                        "connection_state": "2",
                        "error": "0"
                    }, 200)
                    changed_status = True
                    break
                sleep(0.1)

            if not changed_status:
                self.fail("Connection state does not change to 'connecting'")
