import sys
sys.path.append("../../../../tests")
from utility_integration import WrapTest, Env
from utils.general_api import is_package_installed, get_mnfinfo, get_hwinfo
class DataSender(WrapTest):
    url = "/services/data_sender/config"

    def test_basic_crud(self):
        self.crud_test(self.url, {
            ".type": "data_sender",
        }, {
            ".type": "data_sender",
            "enabled":"1",
            "name":"test",
            "data_source":"wifi_scanner",
            "protocol":"mqtt",
            "json_string":"test",
            "json_segment_count":"all",
            "format_non_array":"1",
            "host":"127.0.0.1",
            "port":"420",
            "keepalive":"420",
            "topic":"belenka",
            "client_id":"id",
            "period":"100",
            "device_filtering":"all",
            "filter_mbus_name":"mbusname",
            "filter_dnp3_addr":"65000",
            "filter_dnp3_ip":"127.0.0.1",
            "filter_slave_id":"200",
            "filter_slave_ip":"192.168.1.1",
            "filter_device_mac":"00:00:5e:00:53:af",
            "filter_device_name":"string",
            "stream_id":"testas200",
            "partition_key":"string",
            "api_version":"10",
            "region":"regionas",
            "domain":"192.168.1.1",
            "secret_key":"paslaptis",
            "retry_sending":"1",
            "custom_header":["string", "string2"],
            "tls_enabled":"1",
            "tls_type":"psk",
            "_device_files":"1",
            "psk":"7B316",
            "identity":"value",
            "use_credentials":"1",
            "username":"vitas",
            "password":"test" 
        })

    def test_data_sources(self):
        sources = []

        if is_package_installed(self, "wifi_scanner"):
            sources.append("wifi_scanner")

        if is_package_installed(self, "dnp3"):
            sources.append("dnp3_data")
            sources.append("dnp3_data_flash")

        if is_package_installed(self, "modbus_tcp_master") or is_package_installed(self, "modbus_serial_master"):
            sources.append("modbus_data")

        if is_package_installed(self, "opcua"):
            sources.append("opcua")

        if get_hwinfo(self)["bluetooth"]:
            sources.append("bluetooth_data")

        if "TRB143" in get_mnfinfo(self)["name"]:
            sources.append("mbus")

        x = self.post_data(self.url, {})
        x.assert_code(201)
        sid = x.resp.json()["data"]["id"]

        for source in sources:
            x = self.put_data(self.url + "/" + sid, {
                ".type": "data_sender",
                "data_source": source
            })
            x.assert_data({
                "id": sid,
                ".type": "data_sender",
                "data_source": source
            })

        x = self.delete(f'{self.url}/{sid}')
        x.assert_data({
            "id": sid
        })

    def test_device_filtering(self):
        options = ["all", "slave_id", "slave_ip", "device_mac", "device_name"]

        if is_package_installed(self, "dnp3"):
            options.append("dnp3_addr")
            options.append("dnp3_ip")

        if is_package_installed(self, "modbus_tcp_master") or is_package_installed(self, "modbus_serial_master"):
            options.append("request_name")

        if is_package_installed(self, "opcua"):
            options.append("opcua_name")
            options.append("opcua_name_not")

        if "TRB143" in get_mnfinfo(self)["name"]:
            options.append("mbus_name")
            options.append("mbus_name_not")

        x = self.post_data(self.url, {})
        x.assert_code(201)
        sid = x.resp.json()["data"]["id"]

        for option in options:
            if not "opcua_name" in option:
                x = self.put_data(self.url + "/" + sid, {
                    ".type": "data_sender",
                    "device_filtering": option
                })
                x.assert_data({
                    "id": sid,
                    ".type": "data_sender",
                    "device_filtering": option
                })

            if "opcua_name" in option:
                x = self.put_data(self.url + "/" + sid, {
                    ".type": "data_sender",
                    "device_filtering": option,
                    "filter_opcua_name": "test"
                })
                x.assert_data({
                    "id": sid,
                    ".type": "data_sender",
                    "device_filtering": option,
                    "filter_opcua_name": "test"
                })

        x = self.delete(f'{self.url}/{sid}')
        x.assert_data({
            "id": sid
        })
    
