import sys
sys.path.append("../../../../tests")
import utility_integration as util

class test_dns(util.WrapTest):

    def test_dns_base_functionality(self):
        base_url = "/network/dns/config"
        with self.subTest("multiple_dns_get"):
            x = self.get(base_url)
            x.assert_data([{
                ".type": "dnsmasq",
                "id": "general",
                "rebind_protection": "1",
                "localise_queries": "1",
                "boguspriv": "1"
            }])
        with self.subTest("single_dns_get"):
            x = self.get(base_url + "/general")
            x.assert_data({
                ".type": "dnsmasq",
                "id": "general",
                "rebind_protection": "1",
                "localise_queries": "1",
                "boguspriv": "1"
            })
        with self.subTest("configure_single_dns_section"):
            x = self.put_data(base_url + "/general", {
                "logqueries": "1",
                "localservice": "1",
                "server": [
                    "example.com/example",
                    "example1.com/example",
                    "example2.com/example"
                ],
                "interface":["lan"],
                "rebind_protection":"1",
                "boguspriv":"1",
                "localise_queries": "1",
                "notinterface":["lan"],
                "cachesize":"0"
            })
            x.assert_data({ 
                "cachesize": "0",
                "logqueries": "1",
                "notinterface": [
                    "lan"
                ],
                "id": "general",
                "rebind_protection": "1",
                "boguspriv": "1",
                "localise_queries": "1",
                ".type": "dnsmasq",
                "localservice": "1",
                "interface": [
                    "lan"
                ],
                "server": [
                    "example.com/example",
                    "example1.com/example",
                    "example2.com/example"
                ]
            })
        with self.subTest("check_configured_dns_section"):
            x = self.get(base_url + "/general")
            x.assert_data({
                "cachesize": "0",
                "logqueries": "1",
                "notinterface": [
                    "lan"
                ],
                "id": "general",
                "rebind_protection": "1",
                "boguspriv": "1",
                "localise_queries": "1",
                ".type": "dnsmasq",
                "localservice": "1",
                "interface": [
                    "lan"
                ],
                "server": [
                    "example.com/example",
                    "example1.com/example",
                    "example2.com/example"
                ]
            })
        with self.subTest("clear_dns_configuration"):
            x = self.put_data(base_url + "/general", {
                "logqueries":"",
                "localservice":"",
                "server":[""],
                "interface":[""],
                "notinterface":[""],
                "cachesize":"",
                "rebind_protection": "1",
                "localise_queries": "1",
                "boguspriv": "1"
            })
            x.assert_data({ 
                "id": "general",
                ".type": "dnsmasq",
                "rebind_protection": "1",
                "localise_queries": "1",
                "boguspriv": "1"
            })

    def test_dns_deletion(self):
        x = self.delete("/network/dns/config")
        x.assert_error("Validation", "Section deletion is not allowed", 111, None, None)

    def test_dns_creation(self):
        x = self.post_data("/network/dns/config", {})
        x.assert_error("Validation", "Section creation is not allowed", 108, None, None)

