import unittest
import requests
from utility_integration import Env, WrapTest
from utils.ssh import open_ssh_connection, is_process_running
from utils.general_api import get_modems
import sys
sys.path.append("../../../../tests")
BASE_URL = "/services/events_reporting/config"
OPTIONS_URL = "/services/events_reporting/options"
EMAIL_USERS_URL = "/system/recipients/email_users/config"
SEND_EMAIL_URL = "/services/events_reporting/actions/send_test_email"
EMAIL_GROUP_NAME = "ev_rep_test"


class test_events_reporting(WrapTest):

    def create_email_user(self):
        x = self.post_data(EMAIL_USERS_URL, {
            "name": EMAIL_GROUP_NAME,
            "secure_conn": "1",
            "smtp_ip": "example.com",
            "smtp_port": "123",
            "credentials": "1",
            "username": "test",
            "password": "test",
            "senderemail": "example@teltonika.lt"
        })
        x.assert_code(201)
        self.EMAIL_USER_SID = x.resp.json()["data"]["id"]

    def del_email_user(self):
        x = self.delete(f"{EMAIL_USERS_URL}/{self.EMAIL_USER_SID}")
        x.assert_code(200)

    def create_section(self, body={"enable": "0"}):
        x = self.post_data(BASE_URL, body)
        x.assert_code(201)
        return x.resp.json()["data"]["id"]

    def del_section(self, sid):
        x = self.delete(f"{BASE_URL}/{sid}")
        x.assert_code(200)

    def test_custom_options_validations_saving(self):
        sid = None
        with self.subTest("create section"):
            sid = self.create_section()
            self.create_email_user()

        with self.subTest("main test"):

            # "action" option
            x = self.put_data(f"{BASE_URL}/{sid}", {"action": "sendEmail"})
            x.assert_code(200)
            modems = get_modems(self)
            if len(modems) > 0:
                x = self.put_data(f"{BASE_URL}/{sid}", {"action": "sendSMS"})
                x.assert_code(200)

            x = self.put_data(f"{BASE_URL}/{sid}", {"action": "pp"})
            x.assert_code(422)

            # "*_modem_id" options
            if len(modems) > 1:
                for md in modems:
                    x = self.put_data(
                        f"{BASE_URL}/{sid}", {"info_modem_id": md["id"], "send_modem_id": md["id"]})
                    x.assert_code(200)
            else:
                x = self.put_data(
                    f"{BASE_URL}/{sid}", {"info_modem_id": "bb", "send_modem_id": "ll"})
                x.assert_code(422)

            # "emailgroup" option
            x = self.put_data(f"{BASE_URL}/{sid}",
                              {"emailgroup": "argaergaer"})
            x.assert_code(422)

            x = self.put_data(f"{BASE_URL}/{sid}",
                              {"emailgroup": EMAIL_GROUP_NAME})
            x.assert_code(200)
            self.assertEqual(x.resp.json()["data"]
                             ["emailgroup"], EMAIL_GROUP_NAME)

            section = self.get_section("events_reporting", sid)["values"]
            del section[".name"]
            self.assertEqual(section["emailgroup"], EMAIL_GROUP_NAME)


            x = self.put_data(f"{BASE_URL}/{sid}",
                              {"emailgroup": ""})
            x.assert_code(200)
            section = self.get_section("events_reporting", sid)["values"]
            self.assertNotIn("senderEmail", section)
            self.assertNotIn("password", section)
            self.assertNotIn("userName", section)
            self.assertNotIn("smtpPort", section)
            self.assertNotIn("smtpIP", section)
            self.assertNotIn("secureConnection", section)

            x = self.put_data(f"{BASE_URL}/{sid}",
                              {"emailgroup": "sgfhsgf4115"})
            x.assert_code(422)

        with self.subTest("del section"):
            self.del_section(sid)
            self.del_email_user()

    def test_send_email_check_proccess(self):
        with self.subTest("create email user"):
            self.create_email_user()

        with self.subTest("main test"):
            try:
                api_url = Env.get_api_url()
                x = Env.http.post(f'{api_url}{SEND_EMAIL_URL}', json={"data": {
                    "event": "SMS",
                    "group": EMAIL_GROUP_NAME,
                    "message": "test message",
                    "subject": "testing",
                    "recipients": [
                        "example@teltonika.lt"
                    ]
                }}, timeout=4)

            except:
                with open_ssh_connection() as ssh:
                    self.assertTrue(is_process_running(
                        ssh, "sendmail"), "Expected sendmail to be running")

                    # TODO: can delete this line after issue #9474 is merged
                    ssh.sendline("killall sendmail")

        with self.subTest("del email user"):
            self.del_email_user()

    @unittest.skip("TODO: Can remove this skip after issue #9474 is merged.")
    def test_send_email_failure(self):
        with self.subTest("create email user"):
            self.create_email_user()

        with self.subTest("main test"):
            x = self.post_data(SEND_EMAIL_URL, {
                "event": "SMS",
                "group": EMAIL_GROUP_NAME,
                "message": "test message",
                "subject": "testing",
                "recipients": [
                    "example@teltonika.lt"
                ]
            })
            x.assert_code(422)

        with self.subTest("del email user"):
            self.del_email_user()

    def test_invalid_event(self):
        x = self.post_data(BASE_URL, {"event": "sugma", "eventMark": "bowls"})
        x.assert_code(422)

    def test_events_reporting_daemon_is_running(self):
        sid = None
        with self.subTest("create section"):
            sid = self.create_section({"enable": "1", "event": "Reboot", "eventMark": "all", "action": "sendSMS",
                                      "recipient_format": "single", "message": "test", "telnum": "123456789"})

        with self.subTest("main test"):
            with open_ssh_connection() as ssh:
                self.assertTrue(is_process_running(
                    ssh, "events_reporting"), "Expected events_reporting to be running")

        with self.subTest("del section"):
            self.del_section(sid)

    def test_available_events_saving(self):
        # Using /bulk for faster response because a lot of events exist
        x = self.get(OPTIONS_URL)
        x.assert_code(200)
        self.event_list = {}
        self.event_list = x.resp.json()["data"]["events"]
        s1 = {
            "enable": "",
            "event": "",
            "eventMark": "",
            "action": "",
            "info_modem_id": "",
            "send_modem_id": "",
            "subject": "",
            "message": "",
            "recipient_format": "",
            "telnum": "",
            "group": "",
            "emailgroup": "",
            "recipEmail": "",
            ".type": "rule"
        }

        sid = None
        with self.subTest("create section"):
            sid = self.create_section(s1)

        with self.subTest("main test"):
            bulk = {"data": [
            ]}

            for event, ev_marks in self.event_list.items():
                for ev_mark in ev_marks:
                    bulk["data"].append(
                        {
                            "method": "PUT",
                            "endpoint": BASE_URL,
                            "data": [
                                {"id": sid, "event": event, "eventMark": ev_mark,
                                    ".type": "rule", "enable": "0"}
                            ],
                        }
                    )

            x = self.post("/bulk", bulk)
            res_body = x.resp.json()
            for i, req in enumerate(bulk["data"]):
                res_data = res_body["data"][i]["data"]
                self.assertDictEqual(req["data"][0], res_data[0])

        with self.subTest("del section"):
            self.del_section(sid)
