import sys
sys.path.append("../../../../tests")
from utility_integration import WrapTest

class GPSHTTPS(WrapTest):
    url = "/services/gps/https_general/config/general"
    tavl_url = "/services/gps/https/config"

    def setUp(self):
        if not self.has_gps():
            self.skipTest("device doesn't have gps")

    def has_gps(self):
        response = self.get("/system/device/info")
        board = response.resp.json()["data"]["board"]
        return board["hwinfo"]["gps"]

    def list_tavls(self):
        response = self.get(self.tavl_url)
        response.assert_code(200)
        return response.resp.json()["data"]

    def update_tavl(self, id: str, data: dict):
        self.put_data(f"{self.tavl_url}/{id}", data)

    def test_basic_update(self):
        self.put_data(self.url, {
            "enabled": "1",
            "hostname": "1.2.3.4",
            "id": "https",
            "interval": "1234"
        }).assert_data({
            ".type": "section",
            "enabled": "1",
            "hostname": "1.2.3.4",
            "id": "https",
            "interval": "1234"
        })

    def test_update_tavl(self):
        for tavl in self.list_tavls():
            id = tavl["id"]
            self.put_data(f"{self.tavl_url}/{id}", {
                "enabled": "1"
            }).assert_data({
                "enabled": "1",
                ".type": tavl[".type"],
                "name": tavl["name"],
                "id": tavl["id"],
                "type": tavl["type"]
            })

            self.put_data(f"{self.tavl_url}/{id}", {
                "enabled": "0"
            }).assert_data({
                "enabled": "0",
                ".type": tavl[".type"],
                "name": tavl["name"],
                "id": tavl["id"],
                "type": tavl["type"]
            })
