import io
import utility_integration as util
from utils.ssh import open_ssh_connection, send_cmd
import sys
sys.path.append("../../../../tests")
BASE_URL = "/services/io/juggler/config/general/actions"
TEST_SCRIPT_FILE = "/tmp/test_io_action_script"


class test_io_juggler_action(util.WrapTest):

    def test_script_file_deletion(self):
        x = self.get("/services/io/status/config/din1")
        if x.resp.status_code != 200:
            self.skipTest("Test requires din1 pin")
        with open_ssh_connection() as ssh:
            sid = None
            with self.subTest("setup test"):
                send_cmd(ssh, "touch " + TEST_SCRIPT_FILE)
                x = self.post_data(BASE_URL, {
                                   "ui_name": "din1", "ui_file_path": "path", "type": "script", "path": TEST_SCRIPT_FILE})
                x.assert_code(201)
                sid = x.resp.json()["data"]["id"]

            with self.subTest("test if file is left correctly after changing type"):
                x = self.put_data(f"{BASE_URL}/{sid}", {"type": "reboot"})
                x.assert_code(200)
                res = send_cmd(
                    ssh, f"ls {TEST_SCRIPT_FILE} &> /dev/null ; echo $?")
                self.assertEqual(res.strip(), "0")

            with self.subTest("test if file is left correctly after clearing 'path' option"):
                x = self.put_data(f"{BASE_URL}/{sid}", {"type": "script",
                                  "ui_file_path": "path", "path": TEST_SCRIPT_FILE})
                x.assert_code(200)

                x = self.put_data(f"{BASE_URL}/{sid}", {"type": "script",
                                  "ui_file_path": "", "path": ""})
                x.assert_code(200)

                res = send_cmd(
                    ssh, f"ls {TEST_SCRIPT_FILE} &> /dev/null ; echo $?")
                self.assertEqual(res.strip(), "0")

            with self.subTest("test if file is deleted correctly after changing type"):
                f = io.StringIO("file content\n")
                x = self.send_file(f"{BASE_URL}/{sid}", f, "upload")
                x.assert_code(200)
                uploaded_file_path = x.resp.json()["data"]["path"]

                x = self.put_data(f"{BASE_URL}/{sid}", {"type": "script",
                                  "ui_file_path": "upload", "upload": uploaded_file_path})
                x.assert_code(200)

                x = self.put_data(f"{BASE_URL}/{sid}", {"type": "reboot"})
                x.assert_code(200)

                res = send_cmd(
                    ssh, f"ls {uploaded_file_path} &> /dev/null ; echo $?")
                self.assertEqual(res.strip(), "1")

            with self.subTest("test if file is deleted correctly after clearing 'upload' option"):
                f = io.StringIO("file content\n")
                x = self.send_file(f"{BASE_URL}/{sid}", f, "upload")
                x.assert_code(200)
                uploaded_file_path = x.resp.json()["data"]["path"]

                x = self.put_data(f"{BASE_URL}/{sid}", {"type": "script",
                                  "ui_file_path": "upload", "upload": uploaded_file_path})
                x.assert_code(200)

                x = self.put_data(f"{BASE_URL}/{sid}", {"type": "script",
                                  "ui_file_path": "path", "path":  TEST_SCRIPT_FILE})
                x.assert_code(200)

                res = send_cmd(
                    ssh, f"ls {uploaded_file_path} &> /dev/null ; echo $?")
                self.assertEqual(res.strip(), "1")

            with self.subTest("teardown test"):
                self.delete(f"{BASE_URL}/{sid}")
                res = send_cmd(ssh, "rm " + TEST_SCRIPT_FILE)
