import utility_integration as util
import sys
from response_codes import ResponseCodes
sys.path.append("../../../../tests")


class test_io_juggler_condition(util.WrapTest):

    def test_io_juggler_condition_time_option_validation(self):
        BASE_URL = "/services/io/juggler/config/general/conditions"
        sid = ""
        with self.subTest("create test section"):
            x = self.post_data(BASE_URL, {"type": "minute", "ui_timetype": "0",
                                          "value": "10", "ui_name": "lul"})
            sid = x.resp.json()["data"]["id"]

        with self.subTest("test_io_juggler_condition_time_option_validation"):
            for _type in ["minute", "hour", "weekday", "monthday", "yearday"]:
                x = self.put_data(f"{BASE_URL}/{sid}",
                                  {"type": _type, "value": "5."})
                x.assert_code(422)
                x = self.put_data(f"{BASE_URL}/{sid}",
                                  {"type": _type, "value": "5.1"})
                x.assert_code(422)
                x = self.put_data(f"{BASE_URL}/{sid}",
                                  {"type": _type, "value": "10000"})
                x.assert_code(422)
                x = self.put_data(f"{BASE_URL}/{sid}",
                                  {"type": _type, "value": "5"})
                x.assert_code(200)

            x = self.put_data(f"{BASE_URL}/{sid}", {"type": "hour", "ui_timetype": "1", "interval1": "10:00", "interval2": "11:00", "value": "5"})
            x.assert_code(422)
            x.assert_error(None, "'value' and 'interval' options can not be used simultaneously", ResponseCodes.INVALID_OPT.val())

            x = self.put_data(f"{BASE_URL}/{sid}", {"type": "hour", "ui_timetype": "1", "interval1": "10:00", "interval2": "11:00"})
            x.assert_code(200)

            x = self.put_data(f"{BASE_URL}/{sid}", {"type": "hour", "ui_timetype": "1", "interval1": "12:00", "interval2": "11:00"})
            x.assert_code(200)

            with self.subTest("'value' should not be deleted when 'interval*' is empty"):
                x = self.put_data(f"{BASE_URL}/{sid}", {"type": "hour", "ui_timetype": "0", "interval1": "", "interval2": "", "value": "5"})
                x.assert_code(200)
                self.assertEqual(x.json["data"]["value"], "5")
                self.assertNotIn("interval1", x.json["data"])
                self.assertNotIn("interval2", x.json["data"])

            with self.subTest("'interval*' should not be deleted when 'value' is empty"):
                x = self.put_data(f"{BASE_URL}/{sid}", {"type": "hour", "ui_timetype": "1", "interval1": "11:00", "interval2": "12:00", "value": ""})
                x.assert_code(200)
                self.assertEqual(x.json["data"]["interval1"], "11:00")
                self.assertEqual(x.json["data"]["interval2"], "12:00")
                self.assertNotIn("value", x.json["data"])

        with self.subTest("delete test section"):
            x = self.delete(f"{BASE_URL}/{sid}")
            x.assert_code(200)
