import utility_integration as util
import sys
sys.path.append("../../../../tests")
OUT_PIN_NAME = "dout1"
SMS_RULES_URL = "/services/mobile_utilities/sms_rules/config"
CALL_RULES_URL = "/services/mobile_utilities/call_utilities/rules/config"
BASE_URL = "/services/io/scheduler/config/general/instances"

class test_io_sched_inst(util.WrapTest):

    def test_io_sched_range_validation(self):
        x = self.get("/services/io/status/config/" + OUT_PIN_NAME)
        if x.resp.status_code != 200:
            return self.skipTest("I/O pin '" + OUT_PIN_NAME + "' doesn't exist.")

        id1 = None
        id2 = None
        sms_rules_to_disable = []
        call_rules_to_disable = []
        with self.subTest("setup test"):
            x = self.post_data(BASE_URL, {"pin": OUT_PIN_NAME, "period": "week",
                            "enabled": "0", "start_day": "1", "start_time": "12:00", "end_day": "2", "end_time": "12:00"})
            x.assert_code(201)
            id1 = x.resp.json()["data"]["id"]

            x = self.post_data(BASE_URL, {"pin": OUT_PIN_NAME, "period": "week",
                                        "enabled": "0", "start_day": "1", "start_time": "12:00", "end_day": "3", "end_time": "12:00"})
            x.assert_code(201)
            id2 = x.resp.json()["data"]["id"]

            x = self.get(SMS_RULES_URL)
            if x.resp.status_code == 200:
                for rule in x.resp.json()["data"]:
                    if rule["action"] == "io_set" and rule["enabled"] == "1":
                        sms_rules_to_disable.append(rule)

            x = self.get(CALL_RULES_URL)
            if x.resp.status_code == 200:
                for rule in x.resp.json()["data"]:
                    if rule["action"] == "dout" and rule["enabled"] == "1":
                        call_rules_to_disable.append(rule)

            if len(sms_rules_to_disable) > 0:
                self.put_data(SMS_RULES_URL, [{ "id": s["id"], "enabled": "0" } for s in sms_rules_to_disable ])
            if len(call_rules_to_disable) > 0:
                self.put_data(CALL_RULES_URL, [{ "id": s["id"], "enabled": "0" } for s in call_rules_to_disable ])


        with self.subTest("main test"):
            x = self.put_data(BASE_URL, [
                {"id": id2, "enabled": "1"}
            ])
            x.assert_code(200)

            x = self.put_data(BASE_URL, [
                {"id": id1, "enabled": "1"}, {"id": id2, "enabled": "0"}
            ])
            x.assert_code(200)

            x = self.put_data(BASE_URL, [
                {"id": id1, "enabled": "1"}, {"id": id2, "enabled": "1"}
            ])
            x.assert_code(422)
            resp = x.resp.json()
            self.assertListEqual(
                [{"source": "Validation", "code": 16, "error": "Scheduler interval overlaps with already enabled interval of same output pin",
                    "section": id1},
                {"source": "Validation", "code": 16, "error": "Scheduler interval overlaps with already enabled interval of same output pin",
                    "section": id2}],
                resp["errors"]
            )

            # "period" option
            x = self.put_data(BASE_URL, [
                {"id": id1, "enabled": "1", "period": "week"}, {"id": id2, "enabled": "1", "period": "month"}
            ])
            x.assert_code(422)
            x = self.put_data(BASE_URL, [
                {"id": id1, "enabled": "1", "period": "week", "start_day": "1", "start_time": "12:00", "end_day": "3"},
                {"id": id2, "enabled": "1", "period": "week", "start_day": "4", "start_time": "12:00", "end_day": "5"}
            ])
            x.assert_code(200)

        with self.subTest("teardown test"):
            x = self.delete_data(BASE_URL, [id1, id2])
            x.assert_code(200)
            if len(sms_rules_to_disable) > 0:
                self.put_data(SMS_RULES_URL, [{ "id": s["id"], "enabled": "1" } for s in sms_rules_to_disable ])
            if len(call_rules_to_disable) > 0:
                self.put_data(CALL_RULES_URL, [{ "id": s["id"], "enabled": "1" } for s in call_rules_to_disable ])
