import sys
sys.path.append("../../../../tests")
from utility_integration import WrapTest, Env
from utils.general_api import get_modems
from utils.mobile_utilities import create_phone_group, delete_phone_group
import response_codes as codes

RC = codes.ResponseCodes

class SMSGateway(WrapTest):
    url_post_get = "/services/mobile_utilities/sms_gateway/post_get/config/general"
    url_auto_reply = "/services/mobile_utilities/sms_gateway/auto_reply/config"
    url_email_to_sms = "/services/mobile_utilities/sms_gateway/email_to_sms/config"
    phone_group_name = "test"
    wrong_name = "rampampam"
    modems = []

    def test_post_get(self):
        username = "Test"
        password = "Test"

        x = self.get(self.url_post_get)
        x.assert_code(200)
        data = x.resp.json()["data"]

        if "username" in data:
            username = data["username"]

        if "password" in data:
            password = data["password"]
        
        with self.subTest("Clear Post/Get config"):
            x = self.put_data(self.url_post_get, {
                "enabled": "",
                "allow": "",
                "info_modem_id": "",
                "send_modem_id": "",
                ".type": "post_get"
            })
            x.assert_code(200)

        if len(self.modems) == 0:
            self.modems.extend(get_modems(self))

        with self.subTest("Check 'info_modem_id' and 'send_modem_id' options"):
            if len(self.modems) > 1:
                for modem in self.modems:
                    x = self.put_data(self.url_post_get, {
                        "info_modem_id": modem["id"],
                        "send_modem_id": modem["id"],
                        "username": username,
                        "password": password,
                        "password_confirm": password,
                        ".type": "post_get"
                    })
                    x.assert_data({
                        "info_modem_id": modem["id"],
                        "send_modem_id": modem["id"],
                        "username": username,
                        "password": "set",
                        "id": "general",
                        ".type": "post_get"
                    }, 200)
                x = self.put_data(self.url_post_get, {
                    "info_modem_id": "",
                    "send_modem_id": "",
                    "username": username,
                    "password": password,
                    "password_confirm": password,
                    ".type": "post_get"
                })
                x.assert_code(200)
            elif len(self.modems) == 1:
                x = self.put_data(self.url_post_get, {
                    "info_modem_id": self.modems[0]["id"],
                    ".type": "post_get"
                })
                x.assert_error("info_modem_id", "Only available when device has more than one modem", RC.INVALID_OPT.val())
                x = self.put_data(self.url_post_get, {
                    "send_modem_id": self.modems[0]["id"],
                    ".type": "post_get"
                })
                x.assert_error("send_modem_id", "Only available when device has more than one modem", RC.INVALID_OPT.val())

        with self.subTest("Check 'allow' option"):
            allow_options = ["mobile_data", "sms_delete", "sms_list", "sms_read", "sms_send", "sms_total"]
            x = self.put_data(self.url_post_get, {
                "allow": allow_options,
                "username": username,
                "password": password,
                "password_confirm": password,
                ".type": "post_get"
            })
            x.assert_data({
                "allow": allow_options,
                "username": username,
                "password": "set",
                "id": "general",
                ".type": "post_get"
            }, 200)

    def test_auto_reply(self):
        x = self.get(self.url_auto_reply)
        x.assert_code(200)
        sid = x.resp.json()["data"][0]["id"]

        if len(self.modems) == 0:
            self.modems.extend(get_modems(self))

        with self.subTest("Check 'send_modem_id' option"):
            if len(self.modems) > 1:
                for modem in self.modems:
                    x = self.put_data(f'{self.url_auto_reply}/{sid}', {
                        "send_modem_id": modem["id"]
                    })
                    x.assert_code(200)
                x = self.put_data(f'{self.url_auto_reply}/{sid}', {
                    "send_modem_id": ""
                })
                x.assert_code(200)
            elif len(self.modems) == 1:
                x = self.put_data(f'{self.url_auto_reply}/{sid}', {
                    "send_modem_id": self.modems[0]["id"]
                })
                x.assert_error("send_modem_id", "send_modem_id can only be set if device has more than one modem", RC.INVALID_OPT.val())
        
        with self.subTest("Create phone group"):
            phone_group_sid, _ = create_phone_group(self, self.phone_group_name)

        with self.subTest("Check 'group' option"):
            x = self.put_data(f'{self.url_auto_reply}/{sid}', {
                "mode": "user_group",
                "group": self.phone_group_name
            })
            x.assert_code(200)
            x = self.put_data(f'{self.url_auto_reply}/{sid}', {
                "mode": "everyone",
                "group": ""
            })
            x.assert_code(200)

        with self.subTest("Delete phone group"):
            delete_phone_group(self, phone_group_sid)

        with self.subTest("Check with wrong phone group"):
            x = self.put_data(f'{self.url_auto_reply}/{sid}', {
                "mode": "user_group",
                "group": self.wrong_name
            })
            x.assert_error("group", "user group not found", RC.INVALID_OPT.val())

        with self.subTest("Check if 'tel' option is deleted"):
            x = self.put_data(f'{self.url_auto_reply}/{sid}', {
                "mode": "list_number",
                "tel": ["+3706666666666"]
            })
            x.assert_code(200)
            x = self.put_data(f'{self.url_auto_reply}/{sid}', {
                "mode": "everyone"
            })
            x.assert_code(200)
            if "tel" in x.resp.json()["data"]:
                self.fail("'tel' option was not deleted")

        with self.subTest("Check endpoint with all options"):
            x = self.put_data(f'{self.url_auto_reply}/{sid}', {
                "enabled": "0",
                "every_sms":"1",
                "delete_sms": "1",
                "mode": "everyone",
                "msg": "test test test"
            })

    def test_email_to_sms(self):
        x = self.get(self.url_email_to_sms)
        x.assert_code(200)
        sid = x.resp.json()["data"][0]["id"]

        if len(self.modems) == 0:
            self.modems.extend(get_modems(self))

        with self.subTest("Check 'modem_id' option"):
            if len(self.modems) > 1:
                for modem in self.modems:
                    x = self.put_data(f'{self.url_email_to_sms}/{sid}', {
                        "modem_id": modem["id"]
                    })
                    x.assert_code(200)
                x = self.put_data(f'{self.url_email_to_sms}/{sid}', {
                    "modem_id": ""
                })
                x.assert_code(200)
            elif len(self.modems) == 1:
                x = self.put_data(f'{self.url_email_to_sms}/{sid}', {
                    "modem_id": self.modems[0]["id"]
                })
                x.assert_error("modem_id", "modem_id can only be set if device has more than one modem", RC.INVALID_OPT.val())

        with self.subTest("Clear email to sms"):
            x = self.put_data(f'{self.url_email_to_sms}/{sid}', {
                "enabled": "0",
                "host": "www.example.com",
                "port": "65",
                "username": "test",
                "password": "test",
                "ssl": "0",
                "ssl_verify": "0",
                "limit": "5",
                "time": "min",
                "min": "1",
                "hour": "1",
                "day": "1"
            })
            x.assert_code(200)
        