import sys
sys.path.append("../../../../tests")
from utility_integration import WrapTest, Env
import response_codes as codes
from utils.ssh import open_ssh_connection, send_cmd
from utils.general_api import get_hwinfo, get_modems, get_board
from utils.mobile_utilities import disable_all_sms_rules, enable_scheduler_general, get_io_pins, delete_scheduler_instance, enable_all_sms_rules, create_scheduler_instance
import json
import operator

RC = codes.ResponseCodes

def dateSort(x):
    return x['date']

class SMSRules(WrapTest):
    url_sms_rules = "/services/mobile_utilities/sms_rules/config"
    url_scheduler = "/services/io/scheduler/config/general/instances"
    output_gpio_pins, output_relay_pins = get_io_pins()
    scheduler_instances = []

    def save_scheduler_instances(self):
        x = self.get(self.url_scheduler)
        self.scheduler_instances.extend(x.resp.json()["data"])
        for instance in self.scheduler_instances:
            if "enabled" in instance and instance["enabled"] == "1":
                x = self.put_data(self.url_scheduler + "/" + instance["id"], {
                    "enabled": "0"
                })

    def revert_scheduler_instances(self):
        for instance in self.scheduler_instances:
            if "enabled" in instance and instance["enabled"] == "1":
                x = self.put_data(self.url_scheduler + "/" + instance["id"], {
                    "enabled": "1"
                })

    def test_get_actions(self):
        actions = [
            "reboot",
            "send_status",
            "vpnstatus",
            "mobile",
            "change_mobile_settings",
            "list_of_profile",
            "vpn",
            "change_profile",
            "ssh_access",
            "web_access",
            "ip_unblock",
            "firstboot",
            "userdefaults",
            "fw_upgrade",
            "monitoring_status",
            "uci",
            "rms_status",
            "rms_action",
            "rms_connect",
            "more",
            "exec"
        ]
        hwinfo = get_hwinfo(self)

        for modem in get_modems(self):
            if modem["sim_count"] > 1:
                actions.append("switch_sim")
                break

        if "ios" in hwinfo and hwinfo["ios"]:
            actions.append("iostatus")
            actions.append("io_set")

        if "gps" in hwinfo and hwinfo["gps"]:
            actions.append("gps")
            actions.append("gps_coordinates")

        if "wifi" in hwinfo and hwinfo["wifi"]:
            actions.append("wifi")

        with open_ssh_connection() as ssh:
            cmd_response = send_cmd(ssh, "ls /etc/config/quota_limit &> /dev/null ; echo $?").strip()
            if cmd_response == "0":
                actions.append("data_usage_reset")
                actions.append("data_limit")

            cmd_response = send_cmd(ssh, "ls /etc/config/etherwake &> /dev/null ; echo $?").strip()
            if "ethernet" in hwinfo and hwinfo["ethernet"] and cmd_response == "0" and "eth0" in get_board(self)["network"]["lan"]["device"] :
                actions.append("wol")

        x = self.get("/services/mobile_utilities/sms_rules/options")
        response_actions = x.resp.json()["data"]["actions"]
        response_actions.sort()
        actions.sort()
        self.assertListEqual(response_actions, actions)


    def test_custom(self):

        with self.subTest("Check 'smstext' option validation"):
            x = self.post_data(self.url_sms_rules, {
                "smstext": "smsTextTest"
            })
            sid = x.resp.json()['data']['id']
            x.assert_code(201)
            x = self.post_data(self.url_sms_rules, {
                "smstext": "smsTextTest"
            })
            x.assert_error("smstext", "Such SMS text already exists", RC.INVALID_OPT.val())
            x = self.delete(f'{self.url_sms_rules}/{sid}')
            x.assert_data({
				"id": sid
			})

        with self.subTest("Check 'message' option set"):
            x = self.post_data(self.url_sms_rules, {
                "status_sms": "1",
                "message": ""
            })
            x.assert_error("message", "Option can not be empty", RC.INVALID_OPT.val())

        with self.subTest("Check 'password' option"):
            x = self.post_data(self.url_sms_rules, {
                "authorization": "no",
                "password": "TestPsw69"
            })
            x.assert_error("password", "authorization must be set to 'local'.", RC.INVALID_OPT.val())

            x = self.post_data(self.url_sms_rules, {})
            sid = x.resp.json()['data']['id']
            x.assert_data({
                "password": "unset",
                "authorization": "password",
                "allowed_phone": "all",
                "value": "0",
                ".type": "rule",
                "id": sid
            }, 201)

            x = self.put_data(f'{self.url_sms_rules}/{sid}', {
                "authorization": "local",
                "password": "TestPsw69"
            })
            x.assert_data({
                "password": "set",
                ".type": "rule",
                "allowed_phone": "all",
                "value": "0",
                "id": sid,
                "authorization": "local"
            }, 200)

            x = self.delete(f'{self.url_sms_rules}/{sid}')
            x.assert_data({
				"id": sid
			})

        with self.subTest("Check 'script' option"):
            x = self.post_data(self.url_sms_rules, {
                "action": "exec",
                "script": "TestScript"
            })
            x.assert_error("script", "Script must start with #!/bin/sh and a newline.", RC.INVALID_OPT.val())

    def test_sms_rules_pin_using_scheduler(self):

        if not get_hwinfo(self)["ios"]:
            self.skipTest("For this test need pin in device")

        with self.subTest("Disable all scheduler instances and save them"):
            self.save_scheduler_instances()

        with self.subTest("Disable all SMS rules"):
            disable_all_sms_rules(self)

        enable_scheduler_general(self)

        if len(self.output_gpio_pins) > 0:
            with self.subTest("Create scheduler instance"):
                scheduler_sid = create_scheduler_instance(self, 1, self.output_gpio_pins[0])

            with self.subTest("Create call utilities instance"):
                x = self.post_data(self.url_sms_rules, {
                    "enabled": "1",
                    "action": "io_set",
                    "io": self.output_gpio_pins[0]
                })
                x.assert_error("enabled", f"Unable to enable. Output scheduler instance with '{self.output_gpio_pins[0]}' pin is enabled", 1)

            with self.subTest("Delete scheduler instance"):
                delete_scheduler_instance(self, scheduler_sid)

        with self.subTest("Enable all SMS rules"):
            enable_all_sms_rules(self)

        with self.subTest("Revert scheduler instances 'enabled' option values"):
            self.revert_scheduler_instances()