import sys
import utility_integration as util
import response_codes as codes
sys.path.append("../../../../tests")

class multiusers_groups(util.WrapTest):

    def test_multiusers_groups_base_functionality(self):
        base_url = "/system/groups/config"
        with self.subTest("get_all"):
            x = self.get(base_url)
            x.assert_data(
                [
                    {
                        "target_write": "allow",
                        "target_read": "allow",
                        "id": "root",
                        ".type": "group",
                        "read": [
                            "*"
                        ],
                        "write": [
                            "*"
                        ]
                    },
                    {
                        "target_write": "deny",
                        "target_read": "deny",
                        "id": "admin",
                        ".type": "group",
                        "read": [
                            "system/multiusers/users_configuration",
                            "system/profiles",
                            "system/backup",
                            "system/flashops",
                            "system/cli",
                            "system/wizard",
                            "system/uscripts"
                        ],
                        "write": [
                            "system/multiusers/users_configuration",
                            "system/backup",
                            "system/flashops",
                            "system/cli",
                            "system/wizard",
                            "system/uscripts",
                            "system/profiles",
                            "system/admin/access_control/general"
                        ]
                    },
                    {
                        "target_write": "allow",
                        "target_read": "deny",
                        "id": "user",
                        ".type": "group",
                        "read": [
                            "system/multiusers/users_configuration",
                            "system/flashops",
                            "system/backup",
                            "system/admin/access_control",
                            "system/cli",
                            "system/uscripts",
                            "services/packages",
                            "network",
                            "system/wizard",
                            "services/hotspot/general/userscripts",
                            "services/mobile_utilities/sms_messages/send"
                        ],
                        "write": [
                            "system/multiusers/change_password"
                        ]
                    }
                ]
            )
        with self.subTest("modify_users_group"):
            x = self.put_data(base_url + "/user", {
                "target_write": "allow",
                "target_read": "allow",
                "read": [
                    "services/packages/upload"
                ],
                "write": [
                    "services/packages/upload"
                ]
            })
            x.assert_data({
                "target_write": "allow",
                "target_read": "allow",
                "id": "user",
                ".type": "group",
                "read": [
                    "services/packages/upload"
                ],
                "write": [
                    "services/packages/upload"
                ]
            })
        with self.subTest("get_modified_group"):
            x = self.get(base_url + "/user")
            x.assert_data({
                "target_write": "allow",
                "target_read": "allow",
                "id": "user",
                ".type": "group",
                "read": [
                    "services/packages/upload"
                ],
                "write": [
                    "services/packages/upload"
                ]
            })
        with self.subTest("return_configuration"):
            x = self.put_data(base_url + "/user", {
                "target_write": "allow",
                "target_read": "deny",
                "id": "user",
                ".type": "group",
                "read": [
                    "system/multiusers/users_configuration",
                    "system/flashops",
                    "system/backup",
                    "system/admin/access_control",
                    "system/cli",
                    "system/uscripts",
                    "services/packages",
                    "network",
                    "system/wizard",
                    "services/hotspot/general/userscripts",
                    "services/mobile_utilities/sms_messages/send"
                ],
                "write": [
                    "system/multiusers/change_password"
                ]
            })
            x.assert_data({
                "target_write": "allow",
                "target_read": "deny",
                "id": "user",
                ".type": "group",
                "read": [
                    "system/multiusers/users_configuration",
                    "system/flashops",
                    "system/backup",
                    "system/admin/access_control",
                    "system/cli",
                    "system/uscripts",
                    "services/packages",
                    "network",
                    "system/wizard",
                    "services/hotspot/general/userscripts",
                    "services/mobile_utilities/sms_messages/send"
                ],
                "write": [
                    "system/multiusers/change_password"
                ]
            })

    def test_multiusers_groups_deletion(self):
        x = self.delete("/system/administration/general/config")
        x.assert_error("Validation", "Section deletion is not allowed", codes.ResponseCodes.NO_DELETE.val(), None, None)
    def test_multiusers_groups_creation(self):
        x = self.post_data("/system/administration/general/config", {})
        x.assert_error("Validation", "Section creation is not allowed", codes.ResponseCodes.NO_CREATE.val(), None, None)