import sys
import utility_integration as util
import response_codes as codes
sys.path.append("../../../../tests")

class multiusers_users(util.WrapTest):
    admin_id = ""

    def test_multiusers_users_base_functionality(self):
        base_url = "/system/users/config"
        id = ""
        with self.subTest("create_user"):
            x = self.post_data(base_url, {
                "username": "test",
                "password": "Testas123*",
                "group": "user"
            })
            resp = x.resp.json()
            id = resp["data"]["id"]
            x.assert_data({
                "username": "test",
                "group": "user",
                "id": id,
                ".type": "login"
            }, 201)
        with self.subTest("get_all"):
            x = self.get(base_url)
            resp = x.resp.json()
            for section in resp["data"]:
                if section["username"] == "admin":
                    self.admin_id = section["id"]
            x.assert_data([
                {
                    "username": "admin",
                    "group": "root",
                    ".type": "login"
                },
                {
                    "username": "test",
                    "group": "user",
                    ".type": "login"
                }
            ], 200, {"id"})
        with self.subTest("change_password"):
            x = self.put_data(base_url + "/" + id, {
                "password": "Testas123**",
                "password_confirm": "Testas123**"
            })
            x.assert_data({
                "username": "test",
                "group": "user",
                "id": id,
                ".type": "login"
            })
        with self.subTest("change_group"):
            x = self.put_data(base_url + "/" + id, {
                "group": "admin"
            })
            x.assert_data({
                "username": "test",
                "group": "admin",
                "id": id,
                ".type": "login"
            })
        with self.subTest("delete_user"):
            x = self.delete(base_url + "/" + id)
            x.assert_data({
                "id": id
            })

    def test_multiusers_password_verification(self):
        base_url = "/system/users/config"
        id = ""
        with self.subTest("weak_password"):
            x = self.post_data(base_url, {
                "username": "test",
                "password": "test",
                "group": "user"
            })
            x.assert_error("password",
             "A password of minimum 8 characters and maximum 4094 characters, at least one uppercase letter, one lowercase letter and one number is accepted.", 103, "test", ".anonymous")
        with self.subTest("create_temp_user"):
            x = self.post_data(base_url, {
                "username": "test",
                "password": "Testas123*",
                "group": "user"
            })
            resp = x.resp.json()
            id = resp["data"]["id"]
            x.assert_data({
                "username": "test",
                "group": "user",
                ".type": "login",
                "id": id
            }, 201)
        with self.subTest("password_do_not_match"):
            x = self.put_data(base_url + "/" + id, {
                "password": "testas123**",
                "password_confirm": "miaumiaumiau*"
            })
            resp = x.resp.json()
            errors_body = [
                {
                    "source": "password",
                    "code": 103,
                    "value": "testas123**",
                    "error": "'password' and 'password_confirm' options do not match.",
                    "section": id
                },
                {
                    "source": "password_confirm",
                    "code": 103,
                    "value": "miaumiaumiau*",
                    "error": "'password' and 'password_confirm' options do not match.",
                    "section": id
                }
            ]
            self.assertEqual(resp["errors"], errors_body)
        with self.subTest("delete_temp_user"):
            x = self.delete(base_url + "/" + id)
            x.assert_data({
                "id": id
            })

    def test_multiusers_admin_password(self):
        base_url = "/system/users/config"
        id = ""
        env = util.Env
        with self.subTest("find_admin_id"):
            x = self.get(base_url)
            resp = x.resp.json()
            for section in resp["data"]:
                if section["username"] == "admin":
                    id = section["id"]
        with self.subTest("change_admin_password"):
            x = self.put_data(base_url + "/" + id, {
                "current_password": env.password,
                "password":"Testas123",
                "password_confirm": "Testas123"
            })
            x.assert_data({
                "username": "admin",
                ".type": "login",
                "id": id,
                "group": "root"
            })
        with self.subTest("return_password"):
            x = self.put_data(base_url + "/" + id, {
                "current_password": "Testas123",
                "password": env.password,
                "password_confirm": env.password
            })
            x.assert_data({
                "username": "admin",
                ".type": "login",
                "id": id,
                "group": "root"
            })