import sys
sys.path.append("../../../../tests")
from utility_integration import WrapTest, Env
from utils.ssh import open_ssh_connection, is_process_running, is_process_stopped
from utils.general_api import get_modems
import response_codes as codes

RC = codes.ResponseCodes

class SMPP(WrapTest):
    url = "/services/smpp/config/general"

    def test_basic_update(self):
        self.put_data(self.url, {
            "enabled": "0",
            "username": "usr",
            "password": "pswd",
            "port": "420",
            "modem": ""
        }).assert_data({
            "id": "general",
            ".type": "smpp",
            "enabled": "0",
            "username": "usr",
            "password": "pswd",
            "port": "420",
        })

    def test_check_process(self):
        self.put_data(self.url, {
            "enabled": "0",
        }).assert_code(200)

        with open_ssh_connection() as ssh:
            self.assertTrue(is_process_stopped(ssh, "smppd"), "Expected 'smppd' not to be running")

            self.put_data(self.url, {
                "enabled": "1",
                "username": "usr",
                "password": "pswd",
            }).assert_code(200)

            self.assertTrue(is_process_running(ssh, "smppd"), "Expected 'smppd' to be running")

    def test_modem_option(self):
        modems = get_modems(self)

        if len(modems) > 1:
            for modem in modems:
                x = self.put_data(self.url, {
                    "enabled": "0",
                    "username": "usr",
                    "password": "pswd",
                    "port": "420",
                    "modem": modem["id"]
                })
                x.assert_data({
                    "id": "general",
                    ".type": "smpp",
                    "enabled": "0",
                    "username": "usr",
                    "password": "pswd",
                    "port": "420",
                    "modem": modem["id"]
                }, 200)
            x = self.put_data(self.url, {
                "modem": ""
            })
            x.assert_code(200)
        elif len(modems) == 1:
            x = self.put_data(self.url, {
                "modem": modems[0]["id"],
            })
            x.assert_error("modem", "'modem' option is available only for devices with more than 1 modem.", RC.INVALID_OPT.val())
