import sys
sys.path.append("../../../../tests")
from utility_integration import WrapTest, Env

class test_snmp_settings(WrapTest):
    url_settings = "/services/snmp/settings/config/general"
    url_firewall = "/network/firewall/traffic_rules/config"

    def test_basic_update_snmp_settings(self):
        x = self.put_data(self.url_settings, {
            "enabled": "0",
            "remoteAccess": "0",
            "ipfamily": "ipv4",
            "port": "69",
            "v1mode": "1",
            "v2cmode": "1",
            "v3mode": "1"
        })
        x.assert_data({
            "enabled": "0",
            "id": "general",
            ".type": "agent",
            "remoteAccess": "0",
            "ipfamily": "ipv4",
            "port": "69",
            "v1mode": "1",
            "v2cmode": "1",
            "v3mode": "1"
        })

    def test_check_firewall(self):
        with self.subTest("enable snmp"):
            x = self.put_data(self.url_settings, {
                "enabled": "1",
                "remoteAccess": "1",
                "ipfamily": "ipv4",
                "port": "420",
                "v1mode": "0",
                "v2cmode": "0",
                "v3mode": "0"
            })
            x.assert_code(200)

        with self.subTest("check firewall"):
            x = self.get(self.url_firewall)
            resp = x.resp
            found = False
            for section in resp.json()["data"]:
                if section["name"] == "SNMP_WAN_Access":
                    found = True
                    self.assertEqual(section["enabled"], "1")
                    self.assertEqual(section["target"], "ACCEPT")
                    self.assertEqual(section["proto"], "udp")
                    self.assertListEqual(section["dest_port"], ["420"])

            if not found:
                self.fail("Firewall rule is not created")

        with self.subTest("update snmp"):
            x = self.put_data(self.url_settings, {
                "enabled": "1",
                "remoteAccess": "0",
                "ipfamily": "ipv4",
                "port": "69",
                "v1mode": "0",
                "v2cmode": "0",
                "v3mode": "0"
            })
            x.assert_code(200)

        with self.subTest("check firewall after update"):
            x = self.get(self.url_firewall)
            resp = x.resp
            found = False
            for section in resp.json()["data"]:
                if section["name"] == "SNMP_WAN_Access":
                    found = True
                    self.assertEqual(section["enabled"], "0")
                    self.assertEqual(section["target"], "ACCEPT")
                    self.assertEqual(section["proto"], "udp")
                    self.assertListEqual(section["dest_port"], ["69"])

            if not found:
                self.fail("Firewall rule is not created")