import sys
sys.path.append("../../../../tests")
from utility_integration import WrapTest, Env
from utils.general_api import get_modems
import response_codes as codes

http = Env.http
api_url = Env.get_api_url()
RC = codes.ResponseCodes

class test_snmp_trap_rules(WrapTest):
    url_trap_rules= "/services/snmp/trap_rules/config"
    pins = []

    @classmethod
    def setUpClass(cls):
        x = Env.http.get(Env.get_api_url() + "/services/io/status/config")
        for io in x.json()["data"]:
            cls.pins.append(io["name"])

    def update_state(self, sid, state, pin, io_from="", io_to=""):
        if io_from == "" and io_to =="":
            x = self.put_data(f"{self.url_trap_rules}/{sid}", {
                "type": "iotrap",
                "name": pin,
                "state": state,
                "from": "",
                "to": ""
            }).assert_data({
                "state": state,
                ".type": "trap",
                "name": pin,
                "id": sid,
                "type": "iotrap"
            })

        if not (io_to == "" and io_from ==""):
            x = self.put_data(f"{self.url_trap_rules}/{sid}", {
                "type": "iotrap",
                "name": pin,
                "state": state,
                "from": io_from,
                "to": io_to,
            }).assert_data({
                "state": state,
                ".type": "trap",
                "name": pin,
                "id": sid,
                "type": "iotrap",
                "from": io_from,
                "to": io_to
            })

    def test_snmp_trap_rules_io(self):
        if len(self.pins) == 0:
            self.skipTest("Device does not have IO pins")

        x= self.post(self.url_trap_rules)
        x.assert_code(201)
        sid = x.resp.json()["data"]["id"]

        with self.subTest("check state option"):
            for pin in self.pins:
                if ("din" in pin or "dout" in pin or "iio" in pin or "dio" in pin):
                    self.update_state(sid, "active", pin)
                    self.update_state(sid, "inactive", pin)
                
                if ("relay0" in pin or "relay1" in pin):
                    self.update_state(sid, "open", pin)
                    self.update_state(sid, "closed", pin)

                if ("dwi0" in pin or "dwi1" in pin):
                    self.update_state(sid, "rising", pin)
                    self.update_state(sid, "falling", pin)

                if ("adc0" in pin or "acl0" in pin):
                    self.update_state(sid, "in_range", pin, "5", "10")
                    self.update_state(sid, "out_of_range", pin, "5", "10")
                    x = self.put_data(f"{self.url_trap_rules}/{sid}", {
                        "type": "iotrap",
                        "name": pin,
                        "state": "in_range",
                        "from": "10",
                        "to": "9",
                    })
                    x.assert_error("Validation", "'from' option is bigger or equal to 'to' option", RC.INVALID_OPT.val())
        
        x = self.delete(f"{self.url_trap_rules}/{sid}")
        x.assert_data({
            "id": sid
        })

    def test_snmp_trap_rules_gsm(self):
        modems = get_modems(self)
        if len(modems) == 0:
            self.skipTest("Device does not have GSM")
        
        x= self.post(self.url_trap_rules)
        x.assert_code(201)
        sid = x.resp.json()["data"]["id"]

        if len(modems) > 1:
            x = self.put_data(f"{self.url_trap_rules}/{sid}", {
                "enabled": "1",
                "type": "gsm",
                "name": "conntypetrap",
                "signal": "-69"
            })
            x.assert_error("type", "Missing required option: modem", RC.INVALID_OPT.val())

        for modem in modems:
            x = self.put_data(f"{self.url_trap_rules}/{sid}", {
                "enabled": "0",
                ".type": "trap",
                "name": "signalstrtrap",
                "type": "gsm",
                "signal": "-69",
                "modem": modem["id"]
            }).assert_data({
                "id": sid,
                "enabled": "0",
                ".type": "trap",
                "name": "signalstrtrap",
                "type": "gsm",
                "signal": "-69",
                "modem": modem["id"]
            })

        x = self.delete(f"{self.url_trap_rules}/{sid}")
        x.assert_data({
            "id": sid
        })
