import sys
sys.path.append("../../../../tests")
from utility_integration import WrapTest, Env
from utils.general_api import delete_all, get_serial_device
from utils.ssh import open_ssh_connection, is_process_running, is_process_stopped
from time import sleep

class RSModem(WrapTest):
    url = "/services/rs_modem/config"

    def setUp(self):
        self.device = get_serial_device(self)
        if not self.device:
            self.skipTest("requires serial device")

        delete_all(self, self.url)

    def test_basic_crud(self):
        self.crud_test(self.url, {
            "device": self.device,
            "name": "testing",
            ".type": "modem"
        }, {
            ".type": "modem",
            "baudrate": "9600",
            "ctl_mode": "partial",
            "databits": "8",
            "device": self.device,
            "enabled": "1",
            "flowcontrol": "none",
            "name": "testing",
            "parity": "none",
            "stopbits": "1"
        })

    def test_check_process(self):
        """
            Check if socat process is running after enabling a configuration
        """
        with open_ssh_connection() as ssh:
            self.assertTrue(is_process_stopped(ssh, "socat"), "Expected socat not to be running")

            self.post_data(self.url, {
                "baudrate": "9600",
                "ctl_mode": "partial",
                "databits": "8",
                "device": self.device,
                "enabled": "1",
                "flowcontrol": "none",
                "name": "testing",
                "parity": "none",
                "stopbits": "1"
            })

            self.assertTrue(is_process_running(ssh, "socat"), "Expected socat to be running")
