import sys
sys.path.append("../../../../tests")
import utility_integration as util
from utils.ssh import open_ssh_connection, is_process_running, is_process_stopped

http = util.Env.http
api_url = util.Env.get_api_url()

class ThingWorx(util.WrapTest):
    url = "/services/cloud_solutions/thingworx/config/general"
    url_iface = "/network/interfaces/config"
    iface = None

    @classmethod
    def setUpClass(cls):
        cls.iface = cls.get_first_iface(cls.url_iface)

    @classmethod
    def get_first_iface(self, url):
        x = http.get(api_url + url)
        if x.status_code == 200:
            for iface in x.json()["data"]:
                if "modem" in iface:
                    return iface
        return None    

    def setUp(self):
        if self.iface == None:
            self.skipTest("requires router with modem")

    def test_basic_update(self):
        x = self.put_data(self.url, {
            "appkey": "Test420",
            ".type": "iottw",
            "iface": self.iface["id"],
            "port": "420",
            "thing": "Blatata",
            "enabled": "0",
            "server": "69.69.69.69"
        })
        x.assert_data({
            "appkey": "Test420",
            ".type": "iottw",
            "iface": self.iface["id"],
            "port": "420",
            "thing": "Blatata",
            "enabled": "0",
            "server": "69.69.69.69",
            "id": "general"
        })

    def test_required_options(self):
        with self.subTest("Clear config"):
            x = self.put_data(self.url, {
                "appkey": "",
                ".type": "iottw",
                "iface": "",
                "port": "",
                "thing": "",
                "enabled": "0",
                "server": ""
            })
            x.assert_code(200)

        with self.subTest("Check require"):
            x = self.put_data(self.url, {
                "enabled": "1",
            })
            x.assert_code(422)
            if not len(x.resp.json()["errors"]) == 5:
                self.assertFalse(x.resp.status_code, "Expected five error messages, that tell about required options.")

    def test_check_process(self):
        with open_ssh_connection() as ssh:
            self.assertTrue(is_process_stopped(ssh, "twStreamApp"), "Expected 'twStreamApp' not to be running")
            x = self.put_data(self.url, {
                "appkey": "Test420",
                ".type": "iottw",
                "iface": self.iface["id"],
                "port": "420",
                "thing": "Blatata",
                "enabled": "1",
                "server": "69.69.69.69"
            })
            x.assert_code(200)

            self.assertTrue(is_process_running(ssh, "twStreamApp"), "Expected 'twStreamApp' to be running")
            x = self.put_data(self.url, {
                    "appkey": "",
                    ".type": "iottw",
                    "iface": "",
                    "port": "",
                    "thing": "",
                    "enabled": "0",
                    "server": ""
            })
            x.assert_code(200)