import sys
import utility_integration as util
import response_codes as codes
sys.path.append("../../../../tests")

class troubleshoot_logging_settings(util.WrapTest):

    def test_troubleshoot_logging_base_functionality(self):
        base_url = "/system/logging/config"
        with self.subTest("single_get"):
            x = self.get(base_url + "/general")
            x.assert_data({
                "log_buffer_size":"128",
                "log_type":"circular",
                "id":"general",
                ".type":"system",
                "log_size":"200"
            })
        with self.subTest("multiple_get"):
            x = self.get(base_url)
            x.assert_data([
                {
                    "log_buffer_size":"128",
                    "log_type":"circular",
                    "id":"general",
                    ".type":"system",
                    "log_size":"200"
                }
            ])
        with self.subTest("modify_logging_settings"):
            x = self.put_data(base_url + "/general", {
                "log_buffer_size":"256",
                "log_proto":"udp",
                "log_type":"file",
                "log_size":"400",
                "log_compress":"1"
            })
            x.assert_data({
                "log_buffer_size":"256",
                "log_proto":"udp",
                "log_type":"file",
                "id":"general",
                ".type":"system",
                "log_file":"/usr/var/log/messages",
                "log_size":"400",
                "log_compress":"1",
            })
        with self.subTest("get_modified"):
            x = self.get(base_url + "/general")
            x.assert_data({
                "log_buffer_size":"256",
                "log_proto":"udp",
                "log_type":"file",
                "id":"general",
                ".type":"system",
                "log_size":"400",
                "log_compress":"1",
                "log_file":"/usr/var/log/messages"
            })
        with self.subTest("return_configuration"):
            x = self.put_data(base_url + "/general", {
                "log_buffer_size":"128",
                "log_type":"circular",
                "log_size":"200",
                "log_compress":"",
                "log_hostname":"",
                "log_proto":""
            })
            x.assert_data({
                "log_buffer_size":"128",
                "log_type":"circular",
                "id":"general",
                ".type":"system",
                "log_size":"200"
            })

    def test_troubleshoot_logging_delete_log_file(self):
        base_url = "/system/logging/"
        with self.subTest("delete_log_file"):
            x = self.post(base_url + "actions/delete_log")
            x.assert_error("Request", "Log file is not found.", codes.ResponseCodes.INCORRECT_REQUEST.val())

    def test_troubleshoot_logging_deletion(self):
        x = self.delete("/system/logging/config")
        x.assert_error("Validation", "Section deletion is not allowed", 111, None, None)
    def test_troubleshoot_logging_creation(self):
        x = self.post_data("/system/logging/config", {})
        x.assert_error("Validation", "Section creation is not allowed", 108, None, None)