import sys
sys.path.append("../../../../tests")
import utility_integration as util
import response_codes as codes


class test_wireguard(util.WrapTest):

    def test_wireguard_generate_keys(self): 
        x = self.post("/services/wireguard/actions/generate_keys", None)
        x = x.resp
        self.assertIn("private", x.json()['data'])
        self.assertIn("public", x.json()['data'])

    def test_wireguard_base_functionality(self):
        base_url = "/services/wireguard/config"
        zone = ""
        with self.subTest("create_section"):
            x = self.post_data(base_url, {
                "id": "testWire",
                "enabled": "0",
                "private_key": "wDN6PWV3NOhhXCvjeRG9wWMKAftljpXMs41T464rYXg=",
                "public_key": "QW6ck0GrFJ7Y6C6RzAd2NIW7a055BqsdtrkdbaKAEBQ=",
                "listen_port": "666",
                "addresses":["1.1.1.1/24", "2.2.2.2/25"],
                "metric": "1",
                "mtu": "1000"
            })
            x.assert_data({ "enabled": "0",
                ".type": "interface",
                "mtu": "1000",
                "public_key": "QW6ck0GrFJ7Y6C6RzAd2NIW7a055BqsdtrkdbaKAEBQ=",
                "id": "testWire",
                "metric": "1",
                "listen_port": "666",
                "private_key": "wDN6PWV3NOhhXCvjeRG9wWMKAftljpXMs41T464rYXg=",
                "addresses": ["1.1.1.1/24", "2.2.2.2/25"]
            }, 201)
        with self.subTest("check_firewall_zone"):
            x = self.get("/network/firewall/zones/config")
            resp = x.resp
            found = False
            for section in resp.json()['data']:
                if section['name'] == "wireguard":
                    found = True
                    self.assertEqual(section['input'], "ACCEPT")
                    self.assertEqual(section['forward'], "REJECT")
                    self.assertEqual(section['masq'], "1")
                    self.assertEqual(section['output'], "ACCEPT")
                    zone = section['id']
            if not found:
                self.fail("Zone is not created")
        with self.subTest("configure_section"):
            x = self.put_data(base_url + "/testWire", {
                "enabled": "0",
                "private_key": "QW6ck0GrFJ7Y6C6RzAd2NIW7a055BqsdtrkdbaKAEBQ=",
                "public_key": "wDN6PWV3NOhhXCvjeRG9wWMKAftljpXMs41T464rYXg=",
                "listen_port": "555",
                "addresses":["3.3.3.3/24", "4.4.4.4/25"],
                "metric": "2",
                "mtu": "2000"
            })
            x.assert_data({ 
                "enabled": "0",
                ".type": "interface",
                "mtu": "2000",
                "public_key": "wDN6PWV3NOhhXCvjeRG9wWMKAftljpXMs41T464rYXg=",
                "id": "testWire",
                "metric": "2",
                "listen_port": "555",
                "private_key": "QW6ck0GrFJ7Y6C6RzAd2NIW7a055BqsdtrkdbaKAEBQ=",
                "addresses": ["3.3.3.3/24", "4.4.4.4/25"]
            })
        with self.subTest("get_section"):
            x = self.get(base_url + "/testWire")
            x.assert_data({
                "enabled": "0",
                ".type": "interface",
                "mtu": "2000",
                "public_key": "wDN6PWV3NOhhXCvjeRG9wWMKAftljpXMs41T464rYXg=",
                "id": "testWire",
                "metric": "2",
                "listen_port": "555",
                "private_key": "QW6ck0GrFJ7Y6C6RzAd2NIW7a055BqsdtrkdbaKAEBQ=",
                "addresses": ["3.3.3.3/24", "4.4.4.4/25"]
            })
        with self.subTest("create_peer"):
            x = self.post_data(base_url + "/testWire/peers", {
                "id": "testpeer",
                "public_key": "QW6ck0GrFJ7Y6C6RzAd2NIW7a055BqsdtrkdbaKAEBQ=",
                "allowed_ips": ["1.1.1.1", "2.2.2.2"],
                "description": "description",
                "preshared_key": "QW6ck0GrFJ7Y6C6RzAd2NIW7a055BqsdtrkdbaKAEBQ=",
                "route_allowed_ips":"1",
                "endpoint_host": "example.com",
                "endpoint_port": "666",
                "persistent_keepalive": "10"
            })
            x.assert_data({
                 "endpoint_port": "666",
                "description": "description",
                "id": "testpeer",
                "allowed_ips": [ "1.1.1.1", "2.2.2.2" ],
                "preshared_key": "QW6ck0GrFJ7Y6C6RzAd2NIW7a055BqsdtrkdbaKAEBQ=",
                "public_key": "QW6ck0GrFJ7Y6C6RzAd2NIW7a055BqsdtrkdbaKAEBQ=",
                ".type": "wireguard_testWire",
                "persistent_keepalive": "10",
                "route_allowed_ips": "1",
                "endpoint_host": "example.com"
            }, 201)
        with self.subTest("configure_peer"):
            x = self.put_data(base_url + "/testWire/peers/testpeer", {
                "public_key": "QW6ck0GrFJ7Y6C6RzAd2NIW7a055BqsdtrkdbaKAEBQ=",
                "allowed_ips": ["3.3.3.3", "4.4.4.4"],
                "description": "description2",
                "preshared_key": "QW6ck0GrFJ7Y6C6RzAd2NIW7a055BqsdtrkdbaKAEBQ=",
                "route_allowed_ips":"0",
                "endpoint_host": "example2.com",
                "endpoint_port": "777",
                "persistent_keepalive": "11"
            })
            x.assert_data({
                "endpoint_port": "777",
                "description": "description2",
                "id": "testpeer",
                "allowed_ips": [ "3.3.3.3", "4.4.4.4" ],
                "preshared_key": "QW6ck0GrFJ7Y6C6RzAd2NIW7a055BqsdtrkdbaKAEBQ=",
                "public_key": "QW6ck0GrFJ7Y6C6RzAd2NIW7a055BqsdtrkdbaKAEBQ=",
                ".type": "wireguard_testWire",
                "persistent_keepalive": "11",
                "route_allowed_ips": "0",
                "endpoint_host": "example2.com"
            })
        with self.subTest("get_peer"):
            x = self.get(base_url + "/testWire/peers/testpeer")
            x.assert_data({
                "endpoint_port": "777",
                "description": "description2",
                "id": "testpeer",
                "allowed_ips": [ "3.3.3.3", "4.4.4.4" ],
                "preshared_key": "QW6ck0GrFJ7Y6C6RzAd2NIW7a055BqsdtrkdbaKAEBQ=",
                "public_key": "QW6ck0GrFJ7Y6C6RzAd2NIW7a055BqsdtrkdbaKAEBQ=",
                ".type": "wireguard_testWire",
                "persistent_keepalive": "11",
                "route_allowed_ips": "0",
                "endpoint_host": "example2.com"
            })
        with self.subTest("check_duplicate_peer_validation"):
            x = self.post_data(base_url + "/testWire/peers", {
                "id": "testpeer2",
                "public_key": "QW6ck0GrFJ7Y6C6RzAd2NIW7a055BqsdtrkdbaKAEBQ="
            })
            x.assert_error("public_key", "Public key cannot be the same between peers", codes.ResponseCodes.INVALID_OPT.val(), "QW6ck0GrFJ7Y6C6RzAd2NIW7a055BqsdtrkdbaKAEBQ=", "testpeer2")
        with self.subTest("delete_section"):
            x = self.delete(base_url + "/testWire")
            x.assert_data({
                "id": "testWire"
            })
        with self.subTest("deletion side effects"):
            # Test if peer was deleted with section
            peer = self.get_section("network", "testpeer")
            self.assertEqual(peer, None)

            # Test if peer was deleted with section
            firewall_zone = self.get_section("firewall", zone)
            self.assertEqual(firewall_zone, None)
